package org.ow2.petals.tools.webadmin.util;

import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;

import javax.servlet.http.HttpServletRequest;

/**
 * 
 * @author alouis
 * 
 */
public class ArtifactInstallationHelper {

    private static String ARTIFACTINSTALLATION = "artifact-installation";

    private static String DOWNLOADURI = "download-uri";

    
    /**
     * Return the URI to be used by PEtALS servers to access downloadable
     * artefacts. With http downloads, As the WebConsole can run on multi-IP
     * server, we have to find the correct IP. <br>
     * <li>First, look at the configuration file,</li> <li>then the IP set by
     * web users in their HttpRequest,</li> <li>otherwise, if the HttpRequest IP
     * is localhost, use the JDK InetAddress.getLocalHost().getHostAddress()
     * method</li> <br>
     * For ftp downloads, we retrieve the ftp address from the configuration
     * file
     * 
     * @param httpServletRequest
     * @return
     * @throws URISyntaxException
     */
    public static URI getURI(HttpServletRequest httpServletRequest) throws URISyntaxException {
        // check from configuration file
        URI uri = getURIFromConfigurationFile();
        
        // check from httprequest
        if(uri == null){
            uri = getURIFromHttpRequest(httpServletRequest);
        }
        if(uri == null){
            uri = getURIFromInetAddress(httpServletRequest);
        }
        if(uri == null){
            uri = new URI(httpServletRequest.getRequestURI());
        }
        return uri;
    }

    /**
     * Retrieve URI from config file : ftp or http uri
     * @return
     * @throws URISyntaxException
     */
    public static URI getURIFromConfigurationFile() throws URISyntaxException {
        String s_uri =null;
        URI uri = null;
        try {
            s_uri = WebconsoleConfigHelper.getProperty(ARTIFACTINSTALLATION, DOWNLOADURI);
            if (!StringHelper.isNullOrEmpty(s_uri)) {
                // for http uri, add a "/" at the end of the uri if required
                if(s_uri.toLowerCase().startsWith("http")){
                    if(! "/".equals(s_uri.charAt(s_uri.length()-1))){
                        s_uri+="/";
                    }
                }
                uri = new URI(s_uri);
            }
        } catch (WebConsoleConfigException e) {
            throw new URISyntaxException(s_uri, "URI can not be retrieved from configuration file");
        }
        return uri;
    }
    
    /**
     * Retrieve URI from http request : http uri. If host is localhost, return null.
     * httpRequest parameters are removed from the URL
     * @param httpServletRequest
     * @return
     * @throws URISyntaxException
     */
    public static URI getURIFromHttpRequest(HttpServletRequest httpServletRequest) throws URISyntaxException {
        URI uri = new URI(httpServletRequest.getRequestURL().toString());
        if("localhost".equalsIgnoreCase(uri.getHost())){
            uri = null;
        }else{
            uri = new URI(uri.getScheme(),uri.getUserInfo(),uri.getHost(),uri.getPort(),uri.getPath(),null,null);
        }
        return uri;
    }
    
    /**
     * Replace the Host in the httpRequest URL by the IP retreived from JDK InetAddress API 
     * httpRequest parameters are removed from the URL
     * @return
     * @throws URISyntaxException
     */
    public static URI getURIFromInetAddress(HttpServletRequest httpServletRequest) throws URISyntaxException {
        URI uri =null;
        try {
            String ip = InetAddress.getLocalHost().getHostAddress();
            uri = new URI(httpServletRequest.getRequestURL().toString());
            //replace uri host with ip
            uri = new URI(uri.getScheme(),uri.getUserInfo(),ip,uri.getPort(),uri.getPath(),null,null);
        } catch (UnknownHostException e) {
            throw new URISyntaxException("InetAddress.getLocalhost()","URI can not be construct from machine IP information");
        }
        return uri;
    }
    
}
